
`define bbug 1
`timescale 1ns / 1fs
`default_nettype none
/* for IIGS		*/

module C90cb1		// this version has all z80 & dma, disk code & 6bit code removed & 12bit RGB added (12/10/11) 
  (
   input 	BUS_3M58,		// on a IIgs this is M2B0
   input	 	BUS_7M,
   inout [15:0]  BUS_A,
   inout [7:0]  BUS_D,
	output 	BUS_DMA,
   input 	BUS_DMAIN,
   output   BUS_DMAOUT,
	output   BUS_INH,
   input 	BUS_INTIN,
   output   BUS_INTOUT,
   input    BUS_IOSTROBE,
   input    BUS_IRQ,
   input    BUS_NMI,
   input    BUS_PHASE0,
   input    BUS_PHASE1,
   input    BUS_Q3,
   input    BUS_R_W,
   output   BUS_RDY,
   input    BUS_RESET,
   input    BUS_SYNC,
   input    BUS_USER1,

   output   IDE_CS1,
   output [13:0]   IDE_D,
   input   IDE_RES,
   output   IDE_ACT,   
   input    N_BUS_DEV_SEL,
   input    N_BUS_IO_SEL,
   output [16:0]   SRAM_A,
   inout  [7:0]    SRAM_D,
   output   SRAM_NCS0,
   output   SRAM_NCS1,

   output   SRAM_NOE,
   output   SRAM_NWE,
	
   input 	SYSCLK,
   output [5:0]  RGB,
   output   v_sync,
   output   h_sync,
   input   	SD_DET,
   input    SOFT_TCK,
   input    SOFT_TDI,
   input    SOFT_TMS,
   output   SOFT_TDO
  );

/* Apple clocks sync'd to clk50 */
	reg reset_s;	 
	reg phase0_s;	
	reg Q3_s;		
	reg devsel_s;		
	reg iosel_s;	
	reg iostrobe_s;			
	reg m2b0_s;		
	reg[15:0] bus_a_1;	
	reg[15:0] bus_a_s; 	
	reg[7:0] bus_d_1;		
	reg[7:0] bus_d_s;		
	reg ph1_s;		
	/* ---------------------------- */


	wire bugin;		// used for debugging
	wire 	jumper = IDE_RES & SOFT_TCK; // low if either jumper in place - changed to one & 5/14/11
//	wire jumper = 1'b1;  // fake jumper off
   wire [7:0] 	write_data;
   wire [15:0] video_wr_addr;  
	
   wire 	TxD;

   wire 	video_read;
   wire 	video_we;
   wire 	video_write_SRAM;
	wire pal_we;
	
   assign	IDE_CS1 = SD_DET;

   wire 	CLKFX_OUT;
   wire 	CLKIN_IBUFG_OUT,CLK0_OUT,LOCKED_OUT,LOCKED_OUT_150,CLK0_OUT_150;
   wire 	clk50 = CLKFX_OUT;

	/* --- Apple IIgs must use mask when asserting INH --- */
	/* 	 We don't use INH in this version					 */

	assign 	BUS_INH = 1'bz;	
   assign 	SRAM_NCS0 = 0;
   assign 	SRAM_NCS1 = 1;
   assign 	BUS_DMAOUT = BUS_DMAIN;		// just connect the DMA chain 
	assign 	BUS_INTOUT = BUS_INTIN;		// just connect the interrupt chain
	assign	BUS_DMA = 1'bz;				// we don't use DMA	
   assign 	SOFT_TDO = TxD;
	
   wire [10:0] rom_addr;
   wire [7:0] 	char_data; 		// from font
   wire [7:0] 	video_ram_data;

   wire [16:0] ram_address;	// address to read from video buffer 
   wire 	update_time;
   wire 	auxmem; 	// if 1 means the apple wrote a video address in the bank 1 (aux memory)
	
	/* --- video_wr_addr (16 bit) is the ram address output from the write_clk module --- */
	/* --- on the falling edge of Q3 and rw == 0	 and (user1_s == 0)						 --- */
	/* --- ram_address is the address from the vga_video module to read data from the --- */
	/* --- SRAM or BRAM (on FPGA block RAM)							  							 --- */
	wire write_vid_is_naux = (video_wr_addr > 16'h1FFF) & (video_wr_addr < 16'h6000) & ~auxmem;
	wire read_vid_is_naux = (ram_address[15:0] > 16'h1FFF) & (ram_address[15:0] < 16'h6000) & (ram_address[16] == 1'b0);
	wire[13:0] BRAM_AA_NAUX;
	wire[13:0] BRAM_AB_NAUX;
	wire[7:0] BRAM_DA_NAUX;
	wire[7:0] BRAM_DB_NAUX;

	wire[11:0] BRAM_AA;	// write addr
	wire[11:0] BRAM_AB;	// read addr
	wire[7:0] BRAM_DA;	// write data
	wire[7:0] BRAM_DB;	// read data
	
	wire[7:0] BRAM_AA_PAL;	// write addr
	wire[7:0] BRAM_AB_PAL;	// read addr
	wire[11:0] BRAM_DA_PAL;	// write data
	wire[11:0] BRAM_DB_PAL;	// read data
	
	wire[7:0] pal_addr; // read address
	wire[11:0] pal_data; // read data	

	wire write_vid_is_text = (video_wr_addr > 16'h03FF) & (video_wr_addr < 16'h0C00);
	wire read_vid_is_text = (ram_address[15:0] > 16'h03FF) & (ram_address[15:0] < 16'h0C00);
	wire video_mem = ((video_wr_addr > 16'h1FFF) && (video_wr_addr < 16'h9D00)) ? 1'b1 : 1'b0;	
	wire vid_read_mem = ((ram_address[15:0] > 16'h1FFF) && (ram_address[15:0] < 16'h9D00)) ? 1'b1 : 1'b0;
	wire write_vid_is_gs = ((video_wr_addr > 16'h9CFF) && (video_wr_addr < 16'hA000) && auxmem) ? 1'b1 : 1'b0;
 
	wire write_is_scb = ((video_wr_addr > 16'h9CFF) && (video_wr_addr < 16'h9DC8) && auxmem) ? 1'b1 : 1'b0;
	wire write_is_even = (video_wr_addr[0] == 1'b0) ? 1'b1 : 1'b0; 
	wire write_is_pal =  ((video_wr_addr > 16'h9DFF) && (video_wr_addr < 16'hA000) && auxmem ) ? 1'b1 : 1'b0;

	assign 	SRAM_A = (video_write_SRAM && ~write_vid_is_text && ~write_vid_is_naux  && ~write_vid_is_gs) ? {auxmem, video_wr_addr} : (vid_read_mem  && ~read_vid_is_naux) ? ram_address : 1'bz;

	assign 	SRAM_D = (video_write_SRAM & video_mem & ~write_vid_is_naux && ~write_vid_is_gs) ? write_data : 8'bz;	
	assign 	video_ram_data = (~video_write_SRAM & vid_read_mem & ~read_vid_is_naux) ?  SRAM_D : read_vid_is_text ? BRAM_DB : (~video_write_SRAM & read_vid_is_naux) ? BRAM_DB_NAUX : 8'bz;	

	assign 	BRAM_AA = (video_write_SRAM && write_vid_is_text) ? {auxmem, video_wr_addr[10:0]} : 12'bz; 	// write addr
	assign 	BRAM_AB = (read_vid_is_text) ? {ram_address[16], ram_address[10:0]} : 12'bz;						// read addr
	assign	BRAM_DA = (video_write_SRAM && write_vid_is_text) ? write_data : 8'bz;

	assign 	BRAM_AA_NAUX = (video_write_SRAM && write_vid_is_naux) ? video_wr_addr[13:0] : 14'bz; 	// write
	assign 	BRAM_AB_NAUX = (read_vid_is_naux) ? ram_address[13:0] : 14'bz;						// read addr	
	assign	BRAM_DA_NAUX = (video_write_SRAM && write_vid_is_naux) ? write_data : 8'bz;

reg [7:0] even_pal = 0;
reg [3:0] odd_pal = 0;

	assign	BRAM_AA_PAL = (video_write_SRAM && write_is_pal) ? (video_wr_addr[8:1]) : 8'bz;	
	assign	BRAM_DA_PAL = (video_write_SRAM && write_is_pal) ? {odd_pal, even_pal} : 12'bz;

	assign	BRAM_AB_PAL = pal_addr;
	assign 	pal_data = BRAM_DB_PAL;
	
	assign 	SRAM_NOE = ~video_read;
   assign 	SRAM_NWE = ~video_we;
/* -------------------- scan control bytes ----------------------------------------------	*/	
wire[7:0] scb_addr; // read address
wire[7:0] scb_data; // read data
reg [7:0] scb [199:0];

	always @(posedge clk50)
		begin
		if (video_write_SRAM && write_is_scb) scb [video_wr_addr[7:0]] <= write_data; 
		if(video_write_SRAM && write_is_pal && write_is_even) even_pal <= write_data;
		if(video_write_SRAM && write_is_pal && ~write_is_even) odd_pal <= write_data[3:0];
		end
		
reg [199:0] k;

	initial
		begin
		for (k = 0; k < 200; k = k + 1)
			begin
			scb[k] = 0;
			end
		end

	assign scb_data = scb[scb_addr];

text_buf text_buf (	// buffers text pages 1 & 2, main and aux
	.clka(clk50),
	.wea(video_we),   // enable writing
	.addra(BRAM_AA), 	// write address 12 bit
	.dina(BRAM_DA),  	// write data
	.clkb(clk50),
	.addrb(BRAM_AB), 	// read address 12 bit 
	.doutb(BRAM_DB)); // read data

aux_buf aux_buf(				// buffers main RAM HGR & DHR both pages
	.clka(clk50),
	.wea(video_we),  			// enable writing
	.addra(BRAM_AA_NAUX), 	// write address 14 bit
	.dina(BRAM_DA_NAUX), 	// write data 
	.clkb(clk50),
	.addrb(BRAM_AB_NAUX), 	// read address 14 bit 
	.doutb(BRAM_DB_NAUX)); 	// read data 
	
pal_buf pal_buf (				// buffers palettes 12 bits
									// not used if pal_fixed defined in video module
	.clka(clk50),
	.wea(video_we), 			// enable writing
	.addra(BRAM_AA_PAL), 	// write address 8 bit
	.dina(BRAM_DA_PAL), 		// write data 12 bit
	.clkb(clk50),
	.addrb(BRAM_AB_PAL), 	// read address 8 bit 
	.doutb(BRAM_DB_PAL)); 	// read data 12 bit 

	/* --------------------------------------------------------------------------------- */

	wire reset = ~reset_s | ~LOCKED_OUT;	
	assign 	BUS_RDY = 1'bz;
   wire [5:0]  RGB_from_module;						 
	assign RGB = RGB_from_module;

   vga_video 
     video_module (
		   .clk50(clk50), 
		   .char_data(char_data), 
		   .ram_data(video_ram_data), 
		   .bus_addr(bus_a_s), 
			.bus_data(bus_d_s),	
		   .ph1_s(ph1_s),			
			.phase0_s(phase0_s), 
			.Q3(Q3_s),				
		   .rw(BUS_R_W), 
		   .reset_n(~reset), 
			.user1(BUS_USER1), 	// on a IIgs low means first 128K of RAM
			.iosel_s(iosel_s), 
			.iostrobe_s(iostrobe_s),
			.devsel_s(devsel_s),					
			.m2b0_s(m2b0_s),		// this signals bank 0/1
`ifdef bbug			
			.bugin(bugin),			// added 10/1/10 to pass signal for bug border
`endif

			.scb_data(scb_data),			// input scanline from BRAM
			.scb_addr(scb_addr),			// output scanline address in BRAM
			.pal_data(pal_data),		// input palette even byte from BRAM
			.pal_addr(pal_addr),		// output palette even byte address in BRAM
			
		   .update_time(update_time),
			.auxmem(auxmem),
		   .video_read(video_read),
		   .v_sync(v_sync), 
		   .h_sync(h_sync), 
		   .rom_addr(rom_addr), 
		   .ram_address(ram_address),
			.IDE_D(IDE_D)				// added 12/17/11 for RGB 12 bit
		   );

   write_clk 
     write_clk (
		.clk(clk50), 
		.ph0(BUS_PHASE0), 	// not replaced with phase0_s since sync'd in write_clk 9/25/10
		.Q3(BUS_Q3), 			// sync'd in write_clk 9/25/10
		.rw(BUS_R_W),
		.user1(BUS_USER1), 	// on a IIgs low means first 128K of RAM
		.update_time(update_time), 
		.bus_a_s(bus_a_s), 
		.bus_d_s(bus_d_s),
		.we(video_we),
		.video_write_SRAM(video_write_SRAM),
		.ram_addr(video_wr_addr), 
		.write_data(write_data)
		);

   clk0 clock (
	       .CLKIN_IN(SYSCLK), 
	       .RST_IN(1'b0 /*~BUS_RESET*/),
	       .CLKFX_OUT(CLKFX_OUT), 
	       .CLKIN_IBUFG_OUT(CLKIN_IBUFG_OUT), 
	       .CLK0_OUT(CLK0_OUT), 
	       .LOCKED_OUT(LOCKED_OUT)
	       );

   font font0 (
	       .addr(rom_addr),
	       .dout(char_data),
	       .clk(clk50)
	       );

/* Synchronize Apple slot clocks to CB clock		*/

	reg reset_1 = 0;
	reg phase0_1 = 0;
	reg Q3_1;
	reg devsel_1 = 0;
	reg iosel_1 = 0;
	reg iostrobe_1 = 0;
	reg m2b0_1 = 0;
	reg ph1_1 = 0;
	
   always @(posedge clk50) 
		begin
      Q3_1 <= BUS_Q3; 
      Q3_s <= Q3_1;
		phase0_1 <= BUS_PHASE0;
		phase0_s <= phase0_1;
      ph1_1 <= BUS_PHASE1;
      ph1_s <= ph1_1;
		reset_1 <= BUS_RESET;
		reset_s <= reset_1;
		devsel_1 <= N_BUS_DEV_SEL;
		devsel_s <= devsel_1;
		iosel_1 <=	N_BUS_IO_SEL;
		iosel_s <= iosel_1;
		iostrobe_1 <= BUS_IOSTROBE;
		iostrobe_s <= iostrobe_1;
		m2b0_1 <= BUS_3M58;
		m2b0_s <= m2b0_1;
		bus_a_1 <= BUS_A;
		bus_a_s <= bus_a_1;
		bus_d_1 <= BUS_D;
		bus_d_s <= bus_d_1;
	
		end

`ifdef bbug
	assign bugin = 1'b0;		// replace 1'b0 with a wire/reg to send state to bugborder
`endif 
endmodule
/*------------------------------------------------------------*/
