`timescale 1ns / 1fs
`define bbug 1
`define tester 1
`default_nettype none

/* for IIGS & 12 bit color */
module vga_video (
		  input clk50, 
		  input [7:0] char_data, 	// from font
		  input [7:0] ram_data,  	// from video data SRAM thru cb1.v
		  input [15:0] bus_addr,	// sync'd in cb1.v
		  input [7:0] bus_data,  	// sync'd in cb1.v
		  input ph1_s,					// sync'd in cb1.v
		  input phase0_s,	
		  input Q3,						// sync'd in cb1.v
		  input rw,
		  input reset_n,
		  input user1,					// M2SEL (for GS) on //e pin 39
		  input iosel_s,
		  input iostrobe_s,
		  input devsel_s,								
		  input m2b0_s,				// aka 3M58 is only available on slot 3 on a ROM 01 gs.  1-6 on a ROM 03
`ifdef bbug
			input bugin,				// use this to pass a signal for bugborder
`endif	
			input [7:0] scb_data,		// input scanline from BRAM
			output reg [7:0] scb_addr,		// output scanline address in BRAM
			input [11:0] pal_data,		// input palette even byte from BRAM
			output [7:0] pal_addr,		// output palette even byte address in BRAM
			output update_time,
			output auxmem,
			output video_read,
			output v_sync,
			output h_sync,
			output reg [10:0] rom_addr,  
			output [16:0] ram_address, 
		  output [13:0] IDE_D		// added 12/17/11 for 12 bit RGB
		  );

	reg rw_s;
	reg rw_1;
	reg reset_s;
	reg reset_1;
	reg user1_s;
	reg user1_1;
	reg m2b0_sl;
	reg [9:0]    h_count = 0;	// horz dot count 
   reg [9:0] 	 v_count = 47;	// vert line count
   reg	 clk1;	
   reg 	TEXT = 1;
   reg 	MIXED = 1;
   reg 	PAGE2 = 0;
   reg 	mode_80COL = 0;
	reg WRITERAM_SEL = 0;	// 1 = aux-same as $C005 on, 0 = main - same as $C004 on
	reg LINEAR_SHR = 0;		// $C029 bit 6
 	reg SHR = 0;						
	reg DHR = 0;
	reg monocolor;
	reg [3:0] 	BORDER_COLOR_NUM = 4'b0000;  
	reg [3:0]	TEXT_COLOR_NUM = 4'b1111;	
	reg [3:0]	BACK_COLOR_NUM = 4'b0000;	
/* ----------------- bugborder uses VGA border color as data tester	-----------	*/	
`ifdef bbug
	reg bugborder = 0;		// added 9/30/10 for testing wires & single bit regs
	reg[1:0] bb_enable = 0;	// 0 = disable
	reg[7:0] bb_loc = 0;		// location to test $C0xx
	reg[7:0] bb_loc2 = 0;   // 2nd location to test $C0xx
	reg[7:0] bb_bit = 0;		// bit to test in returned value low nib for 1st location, high nib for 2nd
	reg[7:0] bb_io = 0;		// value returned
	reg[7:0] bb_io2 = 0;    // value returned for 2nd location
	reg bb_stat = 0;			// 0 = location 1, 1 = location 2
	reg bbrw = 0;				// mirrors the apple read/write line (rw) at the time of an I/O access
	reg bb_hit = 0;			// 1 = test address accessed
`endif
/* -----------------------------------------------------------------------------	*/
`ifdef tester
	reg[7:0] test42 = 0;		// input for various tests
`endif
	reg [7:0] 	 shifter7;
   wire [2:0] 	 scanline = v_count[3:1];
   reg [2:0] 	    count7;
   reg [6:0] 	    hc = 0;	
   reg [4:0] 	    vc = 0;

	reg [4:0] 	count28; 				// added for DHR
	reg [7:0]	shr_bytes = 0; 	 	// super hires bytes in 1 scanline	
   reg [15:0] shr_addr = 16'h2000;	// address of bytes for super hires	- 1

   wire 	    GR =  ~(TEXT | ( MIXED &  vc[4] & vc[2])); // not text or mixed and >= line 20 
   reg 		 GR_r, GR_r2;
	wire	hgr;
   reg 		 hor_active_reg;			// gets set to hor_active (line being written) when count7 == 0
   reg [3:0] 	low_color_nibble;		// used to select lores color (16 colors)
   wire [11:0] lowres_color_val12;		// actual 12 bit color
	/* ------ these added 11/2009 to handle border, text & background colors ----------	*/
	reg [3:0] border_number;			// used to select border color (16 colors)
	wire [11:0] border_color12;			// actual 12 bit color (border)
	reg [3:0] text_number;			// used to select text color (16 colors)
	wire [11:0] text_color12;			// actual 12 bit color (text)
	reg [3:0] back_number;			// used to select background color (16 colors)
	wire [11:0] back_color12;			// actual 12 bit color (background)
	/* -------------------------------------------------------------------------------	*/
	
	reg 	HIRES = 0;
	wire	dhires;							// double hires
	wire shires;							// super hires
	reg [11:0] color12;					// hires 12 bit color
	wire [11:0]	dhr_color12;			// double hires color 12 bit
	reg [3:0] dhrcolor_number;			// used to select double hires color (16 colors) 
   reg 	hor_del2;						// gets set to hor_active_reg when count7 == 0
   reg [4:0] flash_count;   
   reg 		 flash, inverse;
   wire 	    flashing = flash ? flash_count[4] : 1'b0;      
   wire 	    disp40_active;
	
	wire Q3_s = Q3;
									
   reg 	vid_del;
	reg  	AltCharSet,  mode_80STORE;	

	assign dhires = DHR & mode_80COL & GR;	
	assign 	shires = SHR & LINEAR_SHR;		// super hires	
   wire 	    h_blank = ( h_count > 639); 	// horz blanking
   wire 	    v_blank = ( v_count > 479);	// vertical blanking	
	 
	wire vp = (h_blank | v_blank) ? 0 : (shifter7[0] ^ (inverse ^ flashing)); 

	assign 	    h_sync = (SHR) ? (h_count < 680) || (h_count > 696) : (h_count < 680) || (h_count > 710);	
   assign 	    v_sync = (v_count < 500) || (v_count > 520);	

	wire hor_active = (SHR == 1) ? (h_count >= 0) & (h_count < 642) : ~mode_80COL ? (h_count > 39) & (h_count < 608) :
									dhires ? (h_count > 48) & (h_count < 613) : (h_count > 47) & (h_count < 612);	

	wire   vert_active =  (SHR == 1) ? (v_count > 47) & ( v_count < 448) : (v_count > 47) & ( v_count < 432); 

	assign disp40_active =  hor_active_reg & hor_del2 & vert_active;

   wire 	    io_access = (bus_addr[15:8] == 8'b11000000);  //$C0xx area being accessed
	wire 	    video_bit = shifter7[0];

	wire allow_shift7 = (mode_80STORE) ? 1'b1: h_count[0];  
   wire load_aux_mem;
	reg 	aux_byte;
	assign 	update_time = (hgr & ~DHR) ?  ((count7 == 1) & (h_count[0] == 1'b1)) : DHR ? (h_count[1] == 1'b1) : (h_count[1] == 1'b0); 
	assign 	video_read = 1'b1;  // SRAM datasheet says /OE can be low for both read and write 

/* --- RGB 12bit using IDE connector and resistors	-------------------------------------------------------	*/
	reg[11:0] shr_color12 = 0;	// 12 bit
	reg [11:0] colors_4096;
	wire [11:0] text12 = (({vp,vp,vp,vp,vp,vp,vp,vp,vp,vp,vp,vp} & text_color12[11:0]) ^ ({~vp,~vp,~vp,~vp,~vp,~vp,~vp,~vp,~vp,~vp,~vp,~vp} & back_color12[11:0]));
	wire [11:0] GR_RGB12 = dhires ? dhr_color12 : (HIRES == 0) ? lowres_color_val12 : video_bit ? color12 : 12'b0; 
	assign IDE_D[13] = v_sync;
	assign IDE_D[12] = h_sync;
	assign IDE_D[11:0] = (h_blank | v_blank)  ? 12'b0 :  ((~disp40_active)) ? border_color12 : /*(test42) ? colors_4096 :*/ (SHR) ? shr_color12 :
							(GR_r2) ? GR_RGB12 : text12 ;
							
	colors_12 colors_12 (
		.border_number(border_number),	// input
		.text_number(text_number),			// input
		.back_number(back_number),			// input
		.dhrcolor_number(dhrcolor_number),	// input	
		.nibble(low_color_nibble),			// input		
		.border_color12(border_color12),	// output
		.text_color12(text_color12),		// output
		.back_color12(back_color12),		// output
		.dhr_color12(dhr_color12),			// output
		.lowres_color_val12(lowres_color_val12)		// output
			);

	reg line_dbl = 0;

/* --------- increments through the dots -------	*/
   always @(posedge clk50) 
		begin
		clk1 <= ~clk1;		
		if(reset_s == 1'b0)
			begin
			h_count <= 0;
			count7 <= 0;
			count28 <= 0;
			v_count <= 0;
			line_dbl <= 0;
			shr_addr <= 0;
			shr_bytes <= 0;
			clk1 <= 0;
			end

		else // not reset
			begin
			if(clk1) 													
				begin				
				if(disp40_active)
					begin										// SHR address calc & line doubler
					if (h_count[1:0] == 2'b10)				
						begin
						shr_addr <= shr_addr + 1;
						if(shr_addr == 16'hA000) shr_addr <= 16'h2000;
						shr_bytes <= shr_bytes + 1;								
						if (shr_bytes == 8'h9F) 
							begin
							if (line_dbl == 0) shr_addr <= shr_addr - shr_bytes; // line doubler
							shr_bytes <= 8'h00;
							line_dbl <= ~line_dbl; // flip doubler flag
							end
						end
					end

				if ((h_count < 6 ) || (count28 == 27) || (h_count > 612 ) ) count28 <= 0;
				else count28 <= count28 + 1;
				if (h_count == 799) 							// end of horizontal count 0-799  
					begin
					h_count <= 0; 								// start over
					count7 <= 0;  								// reset 0-6 counter
					shr_bytes <= 8'h00;
					if ((v_count == 448) || (v_count < 48)) 
						begin
						shr_addr <= 16'h2000;
						line_dbl <= 1'b0;
						end
					if(v_count == 524) v_count <= 0;		// ( last line 0-524 start over)
					else v_count <= v_count +1;			// not last line so start next line
					end  						
				else 												// not end of horizontal count 0-799
					begin
					h_count <= h_count + 1;					// increment the horizontal count 0-799
					if(allow_shift7) 							// every clk1 for 80c, every other clk1 for 40c
						begin
						if(count7 == 6) count7 <= 0;	// end of 0 to 6 count so reset 
						else count7 <= count7 + 1;		// not end so increment	
						end    
					end
				end // clk1
			else // not clk1 
				begin // not clk1
				if(count7 == 3'b0) rom_addr <= {ram_data[7:0], scanline};		// changed to non-blocking assign																									
				end // else: !if(clk1)
			end // not reset
		end // posedge clk50
		
/* --------------- end increments through the dots ------ 	*/  

	wire X_PAGE2 = PAGE2 & ~mode_80STORE; 	

   reg 	high_bit;

   assign load_aux_mem = (mode_80COL & ((~GR_r2 | DHR) & aux_byte)) | SHR;
	assign auxmem = (((mode_80STORE & PAGE2) ^ (WRITERAM_SEL)) | ((m2b0_sl & (user1_s == 0)))); 	  
	wire [5:0] high_vid_addr = (HIRES & GR_r2) ? {1'b0, X_PAGE2, ~X_PAGE2, scanline} : {4'b0000, X_PAGE2, ~X_PAGE2};	// line

	assign     hgr = (HIRES & GR_r2);
						  
	wire [3:0] addr_sum = {~hc[5],~hc[5],hc[4],hc[3]} +  {vc[4],vc[3],vc[4],vc[3]} + 1;		
	wire [15:0] apple_video_addr = (SHR) ? shr_addr : {high_vid_addr, vc[2:0], addr_sum ,hc[2:0]};
   reg [15:0]  apple_video_addr_reg;
	
	assign      ram_address = {load_aux_mem, apple_video_addr_reg};	// this is the one used for getting ram_data from buffer 


	   always @(posedge clk50)
		begin
		if (~disp40_active) 

			begin
`ifdef bbug				
			border_number <= ((BORDER_COLOR_NUM + bugborder) > 15) ? 0 : BORDER_COLOR_NUM + bugborder;
`else				
			border_number <= BORDER_COLOR_NUM;  

`endif
			text_number   <= TEXT_COLOR_NUM;		
			back_number	  <= BACK_COLOR_NUM;		
			end

		if (SHR || LINEAR_SHR && ~TEXT) 
				begin
				hor_active_reg <= hor_active;
				hor_del2 <= hor_active_reg;
				apple_video_addr_reg <= apple_video_addr;
				end			
		else 
      if(clk1 && allow_shift7) 					// every clk1 for 80c, every other clk1 for 40c
			begin						
			if(count7 == 0) 
				begin
				hor_active_reg <= hor_active;
				hor_del2 <= hor_active_reg; 			// was hor_del2 <= hor_active_reg;
				apple_video_addr_reg <= apple_video_addr;				
				end
			end // if(clk1  &&  allow_shift7)
   end
		
	reg [6:0] aux_ram_data0 = 0;
	reg [6:0] main_ram_data1 = 0;
	reg [6:0] aux_ram_data2 = 0;
	reg [6:0] main_ram_data3 = 0;
/* ------------ Super Hires ------------------------- */	
	reg [7:0] shr_data;
	wire shr_resmode = scb_data[7]; 		// 640 if = 1
	wire fill = (shr_resmode == 0) & (scb_data[5] == 1) & (((h_count[1] == 1'b0) && (shr_data[7:4] == 0)) || ((h_count[1] == 1'b1) && (shr_data[3:0] == 0)));
	
//	wire [3:0] palette_num = scb_data[3:0];
	reg [3:0] palette_num = 0;
	reg [3:0] palette_addr = 0;
	wire [8:0] vfix = v_count - 48;
	wire [7:0] a2line = vfix[8:1]; // vfix/2

	always @(posedge clk50)
		begin
		if(SHR & clk1)
			begin
			if(h_count[1:0] == 2'b11) 
				begin
				scb_addr <= a2line;
				palette_addr <= scb_data[3:0];
				shr_data <= ram_data[7:0];
				end
			if (~fill) shr_color12 <= pal_data;
			end
		end
		
	assign pal_addr = (shr_resmode == 1) ? ((h_count[1:0] == 2'b00) ? {palette_addr[3:0], 2'b10, shr_data[7:6]}:
							 (h_count[1:0] == 2'b01) ? {palette_addr[3:0], 2'b11, shr_data[5:4]} :
							 (h_count[1:0] == 2'b10) ? {palette_addr[3:0], 2'b00, shr_data[3:2]} :
								{palette_addr[3:0], 2'b01,shr_data[1:0]}):
								((h_count[1] == 1'b0) ? {palette_addr[3:0], shr_data[7:4]} : {palette_addr[3:0], shr_data[3:0]});

/* --------------------------------------------------------------------	*/
	wire ard0 =  (count28 == 27);
	wire mrd1 = (count28 == 2);
	wire ard2 =  (count28 == 11);
	wire mrd3 = (count28 == 19);
	wire[2:0] left_color = (h_count < 47) ? border_number[3:1] : main_ram_data3[6:4] ;
	

/* DHR */
	always @(posedge clk50)
		begin
		if (dhires)
			begin
			if (ard0) aux_ram_data0 <= ram_data[6:0];
			if (mrd1) main_ram_data1 <= ram_data[6:0];
			if (ard2) aux_ram_data2 <= ram_data[6:0];
			if (mrd3) main_ram_data3 <= ram_data[6:0];
			end
		end 
/* DHR color per pixel */

reg [3:0] c_number = 0;

	always @(posedge clk50)
		begin
			if (monocolor) 
				begin
				if (h_count < 614)
					begin
					case (count28)
					0: c_number <= { aux_ram_data0[0], left_color[2:0] }; 
					1: c_number <= { aux_ram_data0[1:0], left_color[2:1] };
					2: c_number <= { aux_ram_data0[2:0], left_color[2] };
					3: c_number <= aux_ram_data0[3:0];
					4: c_number <= aux_ram_data0[4:1];
					5: c_number <= aux_ram_data0[5:2];
					6: c_number <= aux_ram_data0[6:3];
					7: c_number <= { main_ram_data1[0], aux_ram_data0[6:4] };
					8: c_number <= { main_ram_data1[1:0], aux_ram_data0[6:5] }; 
					9: c_number <= { main_ram_data1[2:0], aux_ram_data0[6] };
					10: c_number <= main_ram_data1[3:0];
					11: c_number <= main_ram_data1[4:1];
					12: c_number <= main_ram_data1[5:2];
					13: c_number <= main_ram_data1[6:3];
					14: c_number <= { aux_ram_data2[0], main_ram_data1[6:4]};
					15: c_number <={ aux_ram_data2[1:0], main_ram_data1[6:5] };			
					16: c_number <= { aux_ram_data2[2:0], main_ram_data1[6] }; 
					17: c_number <= aux_ram_data2[3:0];
					18: c_number <= aux_ram_data2[4:1];
					19: c_number <= aux_ram_data2[5:2];
					20: c_number <= aux_ram_data2[6:3];
					21: c_number <= { main_ram_data3[0], aux_ram_data2[6:4] };
					22: c_number <= { main_ram_data3[1:0], aux_ram_data2[6:5] };
					23: c_number <= { main_ram_data3[2:0], aux_ram_data2[6] };
					24: c_number <= main_ram_data3[3:0]; 
					25: c_number <= main_ram_data3[4:1];
					26: c_number <= main_ram_data3[5:2];
					27: c_number <= main_ram_data3[6:3];		
					default: c_number <= 4'b0000;
					endcase
					dhrcolor_number <=  (c_number > 9) ? 4'b1111 : (c_number > 4) ? 4'b1010 : 4'b0000;
					end
				end
		else	// color
			begin
			if (h_count < 614)
				begin
				case (count28[4:2])
				3'b000: c_number <= aux_ram_data0[3:0]; 
				3'b001: c_number <= { main_ram_data1[0], aux_ram_data0[6:4] };
				3'b010: c_number <= main_ram_data1[4:1];  // >> 1
				3'b011: c_number <= { aux_ram_data2[1:0], main_ram_data1[6:5] };
				3'b100: c_number <= aux_ram_data2[5:2];
				3'b101: c_number <= { main_ram_data3[2:0], aux_ram_data2[6] };
				3'b110: c_number <= main_ram_data3[6:3] /*| {left_color, 1'b0} */;
				3'b111: c_number <= 4'b0000;	// should never happen
				default: c_number <= 4'b0000;
				endcase	
				dhrcolor_number <= c_number;
				end
			else dhrcolor_number <= c_number & border_number;
			end
		end	

	always @(posedge clk50)
		begin		
		if(clk1 && h_count[0] && hgr &&  ~dhires  && ~SHR )
			begin
			vid_del <= video_bit;
			if (((count7) == 0 && ~mode_80COL) || ((count7 == 6) && mode_80COL))
				begin
				shifter7 <= ram_data[6:0];
				high_bit <= ram_data[7];
				end
			else 
				begin
				shifter7 <= {1'b0, shifter7[7:1]};  // right shift 1 bit
				end
			end
		
		else if(clk1 && allow_shift7 && ~hgr  && ~SHR  && ~dhires ) // text, gr
				begin
					if(count7 == 0) 
						begin
						shifter7 <= (~ram_data[7]) ? char_data [7:0] : ~char_data[7:0];
						low_color_nibble <= v_count[3] ? ram_data[7:4] : ram_data[3:0];				
						flash  <= AltCharSet ? 0 : (ram_data[6] & ~ram_data[7]);		
						inverse <=  ~ram_data[7];	
						end
					else	// not count7 == 0 						
						begin
						shifter7 <= {1'b0, shifter7[7:1]};  // right shift 1 bit
						end // else not count7 == 0	 
				end
		end // always @(posedge clk50)

	wire next_vc = clk1 & allow_shift7 & (count7 == 0) & (hc == 64) & ((aux_byte == 0) | ~mode_80COL); 

`define WHITE  6'b111111
`define BLACK  6'b000000
`define GREEN  6'b001000
`define PURPLE 6'b110010
`define RED    6'b110100
`define BLUE   6'b001011  

`define DEEP_RED 		6'b110000
`define BROWN			6'b100100
`define ORANGE			6'b110100
`define DARK_GREEN	6'b000100
`define DARK_GRAY		6'b010101
// `define GREEN			6'b001000	// same as lores light_green same as hires GREEN
`define YELLOW			6'b111100
`define DARK_BLUE		6'b000010	// same as lores deep_blue 
//`define PURPLE			6'b110010	// same as hires PURPLE
`define LIGHT_GRAY	6'b101010
`define PINK			6'b111010
`define MEDIUM_BLUE	6'b000011
`define LIGHT_BLUE	6'b011011
`define AQUAMARINE	6'b001110

	wire       odd_dot = mode_80COL ? ~h_count[1] : h_count[1];

/* hires colors per pixel */ 

   always @(*)
		begin
		if(shifter7[1] || vid_del) // vid_del = video_bit = shifter7[0] 
			begin
			color12 <=12'hFFF;	// 12 bit
			end 
		else 
			begin
				if(odd_dot) 
					begin
						if(high_bit) 
							begin
							color12 <= 12'h22F;
							end
						else 
							begin
							color12 <= 12'hD2D;
							end
					end 
				else 
					begin
						if(high_bit) 
							begin
							color12 <= 12'hF98;
							end
						else 
							begin
							color12 <= 12'h1D0;
							end
					end
			end
		end	
	
	always @(posedge clk50)
		begin
			if(clk1 && allow_shift7) 	// every clk1 for 80c, every other clk1 for 40c
				begin
				if(~(hor_active && vert_active)) // h & v_counts = horz not 40-607, vert 48-431
					begin // not counting when display screen not active (we're in blanking)
					hc <= 7'b0011000;	// ($18 - 24 ) changed to 7 bits 11/10/09
					aux_byte <= GR ? 0 : 1;			// was 1 before aux_byte test
					end // if(~(hor_active && vert_active))    
				else 																		// (hor_active && vert_active)
					begin 															// video is active
					if(count7 == 0)
						begin
						aux_byte <= ~aux_byte; 
						GR_r <= GR;
						GR_r2 <= GR_r;					
						if((aux_byte == 1) || ~mode_80COL)  // always if not 80c othewise every other time
							begin												
							hc <= hc +1;								// increment counter 24 to 63
							end // if((aux_byte == 1) || ~mode_80COL)
						end //if(count7 == 0) 
					end // else: !if(~(hor_active && vert_active))
				end // if (clk1  &&  allow_shift7)
		end // always @ (posedge clk50)


   always @(posedge clk50) 
		begin
			if (~vert_active)
			vc <= 5'b00000;							// changed 11/10/09
			else // vert_active
				begin // vert_active
					if(next_vc) 
						begin 
							if((v_count == 48) || (vc == 24)) 	
								begin
									flash_count <= flash_count +1; // changed to non-blocking assign;
									vc <= 5'b00000; 	
								end   
							else  
								if (v_count[3:0] == 4'b1111)  vc <= vc + 1;  // increment char line every 16 scan lines
						end // next_vc
				end // vert_active
		end // poseedge clk50

   // decoding Apple soft switches 
	always @(negedge Q3_s)
		begin 

		if(phase0_s == 0)
			begin
				m2b0_sl <= m2b0_s; // latch m2b0
			end

		if(ph1_s == 0)
			begin
			if(reset_s == 1'b0) 
				begin

`ifdef tester
				test42 <= 8'b00000000;
`endif
				TEXT <= 1;
				PAGE2 <= 0;
				HIRES <= 0;
				MIXED <= 0;  
				AltCharSet <= 0;
				mode_80COL <= 0;
				mode_80STORE <= 0;
				DHR <= 0;
				SHR <= 0;
				WRITERAM_SEL <= 0;
				/* --- sets initial colors for text, border & background for IIgs --- */
				case(bus_addr)
				16'hC034: BORDER_COLOR_NUM <= bus_data[3:0];
				16'hC022:
					begin
					TEXT_COLOR_NUM <= bus_data[7:4];
					BACK_COLOR_NUM <= bus_data[3:0];
					end
				endcase

				/* ------------------------------------------------------------------- */
				end // if(reset_s == 0) 
			else // not reset
			if(io_access) // && (phase0_s == 1))
				begin
`ifdef bbug
				if((bb_enable[1:0] == 2'b01)  && (bb_loc[7:0] == bus_addr[7:0])) 
					begin
					bb_io[7:0] <= bus_data[7:0];
					bb_stat <= 1'b0;
					bbrw <= rw_s;	// read/write status write = 0, read = 1
					bb_hit <= 1'b1;
					end
				else 
				if((bb_enable[1:0] == 2'b10) && ((bb_loc[7:0] == bus_addr[7:0]) || (bb_loc2[7:0] == bus_addr[7:0])))
					begin
					bb_hit <= 1'b1;
					if (bb_loc[7:0] == bus_addr[7:0]) 
						begin
						bb_io[7:0] <= bus_data[7:0];
						bb_stat <= 1'b0;
						bbrw <= rw_s;	// read/write status write = 0, read = 1
						end
					if (bb_loc2[7:0] == bus_addr[7:0]) 
						begin
						bb_io2[7:0] <= bus_data[7:0];
						bb_stat <= 1'b1;
						bbrw <= rw_s;	// read/write status write = 0, read = 1
						end
					end
				else bb_hit <= 1'b0;
`endif
				if (user1_s == 0)
					begin

//					end
					case(bus_addr[7:0])
					8'h50:      TEXT <= 0;
					8'h51:      TEXT <= 1;
					8'h52:      MIXED <= 0;
					8'h53:      MIXED <= 1;
					8'h54:      PAGE2 <= 0;
					8'h55:      PAGE2 <= 1;   
					8'h56:      HIRES <= 0;
					8'h57:      HIRES <= 1;				  
					8'h5E:		  DHR <= 1;		// double hires on 11/7/09
					8'h5F:		  DHR <= 0;		// double hires off	
					endcase // case(bus_addr[7:0])
					
					if(~rw_s)
						begin           // writes that switch
						case(bus_addr[7:0])
						8'h00: mode_80STORE <= 0; //$C000
						8'h01: mode_80STORE <= 1;	
						8'h0C:  mode_80COL <= 0;
						8'h0D:  mode_80COL <= 1;     	 	
						8'h0E:  AltCharSet <= 0;
						8'h0F:  AltCharSet <= 1; //$C00F
						8'h21: monocolor <= bus_data[7];		// 1 = color disabled for DHR
						8'h29: 
							begin 
							SHR <= bus_data[7];
							LINEAR_SHR <= bus_data[6];
							end
					
						/* --- looks for IIgs border, text & background color changes --- */
						8'h34: 	// $C034 lower 4 bits is color#
							begin
							BORDER_COLOR_NUM <= bus_data[3:0];
							end
`ifdef tester
						8'h42: 
							begin
							test42 <= bus_data[7:0];	// input used for various tests
							end
`endif
`ifdef bbug
						8'h69: bb_enable <= bus_data[1:0]; // pseudo I/O for bugborder input 1 = on
						8'h6A: bb_loc <= bus_data[7:0];  // pseudo I/O for bugborder I/O location to monitor
						8'h6B: bb_loc2 <= bus_data[7:0];	// pseudo I/O for bugborder 2nd I/O loc. to monitor
						8'h6C: bb_bit <= bus_data[7:0];	// pseudo I/O for bugborder bit to test
`endif
						8'h22:	// $C022 high 4 bits = text color# low 4 bits = background color#
							begin
							TEXT_COLOR_NUM <= bus_data[7:4];
							BACK_COLOR_NUM <= bus_data[3:0];
							end
				
						8'h68: 
							begin
							WRITERAM_SEL <= bus_data[4];  // was 11/5/10 RAMWRT <= bus_data[4];	// state reg
							end	
							
/* ------------------------------------------------------------- */
						endcase
						end // if(~rw_s) with user1_s == 0
					end //if (user1_s == 0)
				end // if io_access  
			end // if ph1 == 0
		end // always @ (negedge Q3)

/* sync apple clocks with CB */

   always @(posedge clk50) 
		begin
		rw_1 <= rw;
		rw_s <= rw_1;
		reset_1 <= reset_n;		// this probably shouldn't be done
		reset_s <= reset_1;		// it's sync'd in cb1.v
		user1_1 <= user1;
		user1_s <= user1_1;
		end
`ifdef bbug

/* changes border color to indicate reg or wire value 9/30/10 */
/* bugborder is added to the border_color_num	*/
   always @(posedge clk50) 
		begin
		if(bb_enable[1:0] == 2'b01) 
			begin
			if(bb_hit)
				begin
				case (bb_bit[3:0])
				8'h00: bugborder <= bb_io[0];
				8'h01: bugborder <= bb_io[1];
				8'h02: bugborder <= bb_io[2];
				8'h03: bugborder <= bb_io[3];
				8'h04: bugborder <= bb_io[4];
				8'h05: bugborder <= bb_io[5];
				8'h06: bugborder <= bb_io[6];
				8'h07: bugborder <= bb_io[7];
				8'h08: bugborder <= 1'b1;
				default: bugborder <= 0;
				endcase
				end
			end
		else 
		if(bb_enable[1:0] == 2'b10)
			begin
			if ((bb_stat == 1'b0) && bb_hit)
				begin
				case (bb_bit[3:0])
				8'h00: bugborder <= bb_io[0];
				8'h01: bugborder <= bb_io[1];
				8'h02: bugborder <= bb_io[2];
				8'h03: bugborder <= bb_io[3];
				8'h04: bugborder <= bb_io[4];
				8'h05: bugborder <= bb_io[5];
				8'h06: bugborder <= bb_io[6];
				8'h07: bugborder <= bb_io[7];
				8'h08: bugborder <= bb_stat;	// location 1 accessed
				default: bugborder <= 1'b0;		// off state if 8'h08
				endcase
				end
			else 
			if ((bb_stat == 1'b1) && bb_hit)
				begin
				case (bb_bit[7:4])
				8'h00: bugborder <= bb_io2[0];
				8'h01: bugborder <= bb_io2[1];
				8'h02: bugborder <= bb_io2[2];
				8'h03: bugborder <= bb_io2[3];
				8'h04: bugborder <= bb_io2[4];
			 	8'h05: bugborder <= bb_io2[5];
				8'h06: bugborder <= bb_io2[6];
				8'h07: bugborder <= bb_io2[7];
				8'h08: bugborder <= bb_stat;	// location 2 accessed
				default: bugborder <= 1'b1;
				endcase
				end
			end
		else	
		bugborder <= 1'b0;			// any single bit reg or wire can be place here
		end
`endif
endmodule
/* --------------------------------------------------------------------	*/
